﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.BusinessLogicsContracts;
using FishFactoryContracts.SearchModels;
using FishFactoryDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FishFactoryView
{
	public partial class FormShop : Form
	{
		private readonly ILogger _logger;
		private readonly IShopLogic _logic;
		private int? _id;
		private Dictionary<int, (ICannedModel, int)> _shopListCanned;
		public int Id { set { _id = value; } }

		public FormShop(ILogger<FormShop> logger, IShopLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
			_shopListCanned = new();
		}

		private void FormShop_Load(object sender, EventArgs e)
		{

			if (_id.HasValue)
			{
				_logger.LogInformation("Загрузка магазина");
				try
				{
					var view = _logic.ReadElement(new ShopSearchModel
					{
						Id = _id.Value
					});
					if (view != null)
					{
						textBoxShop.Text = view.ShopName;
						textBoxAddress.Text = view.Address;
						dateTimePicker.Text = view.DateOpening.ToString();
						_shopListCanned = view.ListCanned ?? new Dictionary<int, (ICannedModel, int)>();
						LoadData();
					}
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка загрузки магазина");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
						MessageBoxIcon.Error);
				}
			}
		}

		private void LoadData()
		{
			_logger.LogInformation("Загрузка консервы магазина");
			try
			{
				if (_shopListCanned != null)
				{
					dataGridView.Rows.Clear();
					foreach (var elem in _shopListCanned)
					{
						dataGridView.Rows.Add(new object[] { elem.Key, elem.Value.Item1.CannedName, elem.Value.Item2 });
					}
				}
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки консерв магазина");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxShop.Text))
			{
				MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(textBoxAddress.Text))
			{
				MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Сохранение магазина");
			try
			{
				var model = new ShopBindingModel
				{
					Id = _id ?? 0,
					ShopName = textBoxShop.Text,
					Address = textBoxAddress.Text,
					DateOpening = dateTimePicker.Value.Date,
					ListCanned = _shopListCanned
				};
				var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
				if (!operationResult)
				{
					throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка сохранения магазина");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
