﻿using FishFactoryContracts.BusinessLogicsContracts;
using FishFactoryContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FishFactoryView
{
	public partial class FormShopCanned : Form
	{
		private readonly ILogger _logger;
		private readonly IShopLogic _shopLogic;
		private readonly ICannedLogic _CannedLogic;
		private readonly List<ShopViewModel>? _listShops;
		private readonly List<CannedViewModel>? _listCanned;

		public FormShopCanned(ILogger<FormShopCanned> logger, IShopLogic shopLogic, ICannedLogic CannedLogic)
		{
			InitializeComponent();
			_shopLogic = shopLogic;
			_CannedLogic = CannedLogic;
			_logger = logger;
			_listShops = shopLogic.ReadList(null);
			if (_listShops != null)
			{
				comboBoxShop.DisplayMember = "ShopName";
				comboBoxShop.ValueMember = "Id";
				comboBoxShop.DataSource = _listShops;
				comboBoxShop.SelectedItem = null;
			}

			_listCanned = CannedLogic.ReadList(null);
			if (_listCanned != null)
			{
				comboBoxCanned.DisplayMember = "CannedName";
				comboBoxCanned.ValueMember = "Id";
				comboBoxCanned.DataSource = _listCanned;
				comboBoxCanned.SelectedItem = null;
			}
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (comboBoxShop.SelectedValue == null)
			{
				MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (comboBoxCanned.SelectedValue == null)
			{
				MessageBox.Show("Выберите консерву", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Добавление консервы в магазин");
			try
			{
				var сanned = _CannedLogic.ReadElement(new()
				{
					Id = (int)comboBoxCanned.SelectedValue
				});
				if (сanned == null)
				{
					throw new Exception("Не найдено консерва. Дополнительная информация в логах.");
				}
				var resultOperation = _shopLogic.AddCannedInShop(
					 model: new() { Id = (int)comboBoxShop.SelectedValue },
					 canned: сanned,
					 count: (int)numericUpDownCount.Value
				);
				if (!resultOperation)
				{
					throw new Exception("Ошибка при добавлении. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение",
				MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка сохранения консервы");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
