using FishFactoryContracts.BindingModels;
using FishFactoryContracts.BusinessLogicsContracts;
using FishFactoryDataModels.Enums;
using Microsoft.Extensions.Logging;
using System.Windows.Forms;

namespace FishFactoryView
{
	public partial class MainForm : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;
		public MainForm(ILogger<MainForm> logger, IOrderLogic orderLogic)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
		}
		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}
		private void LoadData()
		{
			_logger.LogInformation(" ");
			try
			{
				var list = _orderLogic.ReadList(null);
				if (list != null)
				{
					dataGridView.DataSource = list;
					dataGridView.Columns["CannedId"].Visible = false;
					dataGridView.Columns["CannedName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
				}
				_logger.LogInformation(" ");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "  ");
				MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ComponentToolStripMenuItem_Click(object sender, EventArgs
	   e)
		{
			var service =
		   Program.ServiceProvider?.GetService(typeof(FormComponents));
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}
		}
		private void CannedToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormCanneds));
			if (service is FormCanneds form)
			{
				form.ShowDialog();
			}
		}
		private void ButtonCreateOrder_Click(object sender, EventArgs e)
		{
			var service =
		   Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}
		private void ButtonTakeOrderInWork_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation(" {id}.    ' '", id);
				try
				{
					var operationResult = _orderLogic.TakeOrderInWork(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("  .    .");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "    ");
					MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonOrderReady_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation(" {id}.    ''",
			   id);
				try
				{
					var operationResult = _orderLogic.FinishOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("  .    .");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "    ");
					MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonIssuedOrder_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation(" {id}.    ''", id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("  .    .");
					}
					_logger.LogInformation(" {id} ", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "    ");
					MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
				   MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
		private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
		{
			return new OrderBindingModel
			{
				Id = id,
				CannedId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["CannedId"].Value),
				Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
				Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
				Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
				DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
			};
		}
		private void ShopsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormShops));
			if (service is FormShops form)
			{
				form.ShowDialog();
			}
		}
		private void ButtonAddCannedInShop_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormShopCanned));
			if (service is FormShopCanned form)
			{
				form.ShowDialog();
			}
		}
	}
}